/*
 * =====================================================================================
 *
 *       Filename:  othello.c
 *
 *    Description:  Handle the othello board content
 *
 *        Version:  1.0
 *        Created:  25/04/2017 15:16:08
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Jerome Benoit (fraggle), jerome.benoit@piment-noir.org
 *   Organization:  Piment Noir
 *
 * =====================================================================================
 */

#include <stdlib.h>
#include <stdbool.h>

#include "othello.h"

unsigned int current_player(unsigned int round_count) {
   
    if (round_count % 2 != 0) {
        return player_two;
    } else {
        return player_one;
    }
}

/* for consitency with ncurses, the board coordinates are in the following order:
 * O-x-->
 * |
 * y
 * |
 * v 
 * The origin O has (1, 1) coordinates */

unsigned int get_box_value(int y, int x, unsigned int pawn_array[board_size][board_size]) {
   
    return pawn_array[y-1][x-1];
}

bool is_box_type(int y, int x, unsigned int pawn_array[board_size][board_size], unsigned int type) {

    if (type > 2) {
        return NULL;
    }
    if (get_box_value(y, x, pawn_array) == type) {
        return true;
    } else {
        return false;
    }
}

/* helper function to set a value != empty at the (y, x) in the pawns array */
int** set_pawn(int y, int x, unsigned int type, unsigned int pawn_array[board_size][board_size]) {

    if (is_box_type(y, x, pawn_array, empty)) {
        pawn_array[y-1][x-1] = type;
        return pawn_array;
    } else {
        return NULL;
    }
}

static int** zero_pawns(unsigned int pawn_array[board_size][board_size]) {

    for (unsigned int i = 1; i <= board_size; i++) {
        for (unsigned int j = 1; j <= board_size; j++) {
            pawn_array = set_pawn(i, j, empty, pawn_array);
         }
    }
    return pawn_array;
}

/* set the pawns in the start position */
int** init_pawns(unsigned int pawn_array[board_size][board_size]) {

    pawn_array = zero_pawns(pawn_array);
    pawn_array = set_pawn(5, 4, black, pawn_array);
    pawn_array = set_pawn(4, 5, black, pawn_array);
    pawn_array = set_pawn(4, 4, white, pawn_array);
    pawn_array = set_pawn(5, 5, white, pawn_array);
    return pawn_array;
}

unsigned int count_pawn_type(unsigned int pawn_array[board_size][board_size], unsigned int type) {
    unsigned int count = 0;

    if (type > 2) {
        return -1;
    }
    for (unsigned int i = 1; i <= board_size; i++) {
        for (unsigned int j = 1; j <= board_size; j++) {
            if (get_box_value(i, j, pawn_array) == type) {
                count++;
            }
        }
    }
    return count;
}

bool is_valid_input(int y, int x, unsigned int pawn_array[board_size][board_size]) {
    
    /* FIXME: separate the tests to permit to print explicit error messages */
    if ((y > 0 && y < board_size + 1) && \
            (x > 0 && x < board_size + 1) && \
            is_box_type(y, x, pawn_array, empty)) {
        return true;
    } else {
        return false;
    }
}

bool is_board_full(unsigned int pawn_array[board_size][board_size]) {

    for (unsigned int i = 1; i <= board_size; i++) {
        for (unsigned int j = 1; j <= board_size; j++) {
            if (is_box_type(i, j, pawn_array , empty)) {
                return false;
            }
        }
    }
    return true;
}

void status_pawn(int y, int x, unsigned int pawn_array[board_size][board_size]) {

}

bool is_legal_box(int y, int x, int player, unsigned int pawn_array[board_size][board_size]) {

}

bool reverse_box(unsigned int pawn_array[board_size][board_size]) {

}
